#include "keyun.h"

/*
keyun C-1.022
Reading records to arrange lines in descending order, for example
Copyright 2019 Sogaya

gcc (GCC) 3.4.2 (mingw-special)
Borland C++ 5.5.1 for Win32

*/

extern int check_ID[LIMIT_OF_MY_MEMORY];

int main(int argc, char *argv[]) {
    const BOOL WITH_COMMENTS = TRUE;
    const BOOL WITHOUT_COMMENTS = FALSE;
    const char SETTING_TEXT[] = "manager.txt";
    char EUC_JP_TERM[] = "-e";
    char HELP[] = "--help";
    char NEW_FILE[] = "-o";
    char STANDARD_OUT[] = "-o-";
    char UTF_8_TERM[] = "-8";
    char VERSION_WITH_COMMENTS[] = "-VC";
    char VERSION_WITHOUT_COMMENTS[] = "-V";

    BOOL add_ID = FALSE;
    BOOL carriage_return = TRUE;
    BOOL case_sensitive = TRUE;
    BOOL descending = FALSE;
    BOOL loud = TRUE;
    BOOL new_file = FALSE;
    BOOL remove_temporary_binary_file = TRUE;
    BOOL report_ID = FALSE;
    BOOL unique_summary = FALSE;
    char file_name[ FILE_NAME_SIZE + 1 ];
    char new_name[ FILE_NAME_SIZE + 1 ];
    int exit_code = 0;
    int i, h;
    int line_count = 0;
    int manager = PHRASES;
    int record_size;

    memset(new_name, 0, FILE_NAME_SIZE + 1);

    file_name[FILE_NAME_SIZE] = '\0';
    strncpy(file_name, TEMPORARY_BINARY, FILE_NAME_SIZE);

/* Acquiring parameters from SETTING_TEXT */
    acquire_parameters_from_text(SETTING_TEXT, file_name, new_name, &line_count, &manager, &record_size, &add_ID, &case_sensitive, &report_ID, &carriage_return, &descending, &loud, &remove_temporary_binary_file, &unique_summary);

    for(i = 1; i < argc; ++i) {
        if(! strcmp(DESCENDING, argv[i]) ) {
            descending = TRUE;
        } else if(! strcmp(CRLF, argv[i]) ) {
            carriage_return = TRUE;
        } else if(! strcmp(DO_NOT_REMOVE_0DH, argv[i]) ) {
            carriage_return = TRUE;
        } else if(! strcmp(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE, argv[i]) ) {
            remove_temporary_binary_file = FALSE;
        } else if(! strcmp(EUC_JP_TERM, argv[i]) ) {
            carriage_return = FALSE;
        } else if(! strcmp(HELP, argv[i]) ) {
            version_guide(VERSION, WITH_COMMENTS, loud);
        } else if(! strcmp(LINE_FOR_EACH_UNIQUE_SUMMARY, argv[i]) ) {
            unique_summary = TRUE;
        } else if(! strcmp(NEW_FILE, argv[i]) ) {
            new_file = TRUE;
        } else if(! strcmp(NOT_CASE_SENSITIVE, argv[i]) ) {
            case_sensitive = FALSE;
        } else if(! strcmp(LF, argv[i]) ) {
            carriage_return = FALSE;
        } else if(! strcmp(LINE_FEED, argv[i]) ) {
            carriage_return = FALSE;
        } else if(! strcmp(SILENT, argv[i]) ) {
            loud = FALSE;
        } else if(! strcmp(STANDARD_OUT, argv[i]) ) {
            memset(new_name, 0, FILE_NAME_SIZE);
        } else if(! strcmp(UTF_8_TERM, argv[i]) ) {
            carriage_return = FALSE;
        } else if(! strcmp(VERSION_WITH_COMMENTS, argv[i]) ) {
            version_guide(VERSION, WITH_COMMENTS, loud);
        } else if(! strcmp(VERSION_WITHOUT_COMMENTS, argv[i]) ) {
            version_guide(VERSION, WITHOUT_COMMENTS, loud);
        } else {
            if(new_file) {
                new_name[FILE_NAME_SIZE] = '\0';
                strncpy(new_name, argv[i], FILE_NAME_SIZE);
                new_file = FALSE;
            } else {
                file_name[FILE_NAME_SIZE] = '\0';
                strncpy(file_name, argv[i], FILE_NAME_SIZE);
            }

        } /* if */

    } /* for(i = 1; i < argc; ++i) */

    if(unique_summary) {
        exit_code = arranging_unique_lines_to_produce_text(file_name, new_name, line_count, manager, record_size, add_ID, carriage_return, case_sensitive, descending);
    } else {
        exit_code = arranging_to_produce_text(file_name, new_name, line_count, manager, record_size, add_ID, carriage_return, case_sensitive, descending);
    }

    if(remove_temporary_binary_file) {
        remove(TEMPORARY_BINARY);
    }

    if(report_ID) {
        for(h = 0; h < LIMIT_OF_MY_MEMORY && check_ID[h] > 0; ++h) {
            fprintf(stdout, "Line-%d\n", check_ID[h]);
        }

    }

    return exit_code;
}

void acquire_parameters_from_text(const char *parameters_text, char *file_name, char *new_name, int *line_count, int *manager, int *record_size, BOOL *add_ID, BOOL *case_sensitive, BOOL *report_ID, BOOL *carriage_return, BOOL *descending, BOOL *loud, BOOL *remove_temporary_binary_file, BOOL *unique_summary) {
    char *p;
    char *stop;
    char c[C_SIZE];
    FILE *setting = fopen(parameters_text, "rt");
    int k;
    int line_number = 0;

    if(setting != NULL) {
        for(;;) {
            p = fgets(c, C_SIZE, setting);
            if(p == NULL) {
                break;
            } else {
                k = strcspn(c, "\n'|\t");
                c[k] = '\0';
            }

            line_number += 1;
            switch(line_number) {
            case 1:
                *line_count = (int)strtol(p, &stop, 10); /* 520 lines */
                break;
            case 2:
                *manager = (int)strtol(p, &stop, 10); /* 0: pharases. 1: integers. 2: decimals. */
                break;
            case 3:
                *record_size = (int)strtol(p, &stop, 10); /* 3072 bytes */
                break;
            case 4:
                if(! strcmp(ADD_ID, c) ) { /* -# */
                    *add_ID = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *add_ID = FALSE;
                } else {
                    *add_ID = TRUE;
                }

                break;
            case 5:
                if(! strcmp(NOT_CASE_SENSITIVE, c) ) { /* -f */
                    *case_sensitive = FALSE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *case_sensitive = TRUE;
                } else {
                    *case_sensitive = FALSE;
                }

                break;
            case 6:
                if(! strcmp(REPORT_ID, c) ) { /* -! */
                    *report_ID = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *report_ID = FALSE;
                } else {
                    *report_ID = TRUE;
                }

                break;
            case 7:
                file_name[FILE_NAME_SIZE] = '\0';
                strncpy(file_name, c, FILE_NAME_SIZE); /* binary.tmp */
                break;
            case 8:
                new_name[FILE_NAME_SIZE] = '\0';
                strncpy(new_name, c, FILE_NAME_SIZE); /* encoded.txt */
                break;
            case 9:
                if(! strcmp(CRLF, c) ) { /* -CRLF */
                    *carriage_return = TRUE;
                } else if(! strcmp(DO_NOT_REMOVE_0DH, c) ) { /* -L- */
                    *carriage_return = TRUE;
                } else if(! strcmp(LF, c) ) { /* -LF */
                    *carriage_return = FALSE;
                } else if(! strcmp(LINE_FEED, c) ) { /* -L */
                    *carriage_return = FALSE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *carriage_return = FALSE;
                } else {
                    *carriage_return = TRUE;
                }

                break;
            case 10:
                if(! strcmp(DESCENDING, c) ) { /* -r */
                    *descending = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *descending = FALSE;
                } else {
                    *descending = TRUE;
                }

                break;
            case 11:
                if(! strcmp(SILENT, c) ) { /* -s */
                    *loud = FALSE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *loud = TRUE;
                } else {
                    *loud = FALSE;
                }

                break;
            case 12:
                if(! strcmp(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE, c) ) { /* -y */
                    *remove_temporary_binary_file = FALSE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *remove_temporary_binary_file = TRUE;
                } else {
                    *remove_temporary_binary_file = FALSE;
                }

                break;
            case 13:
                if(! strcmp(LINE_FOR_EACH_UNIQUE_SUMMARY, c) ) { // -u
                    *unique_summary = TRUE;
                } else if(! strcmp(ZERO, c) ) { // 0
                    *unique_summary = FALSE;
                } else {
                    *unique_summary = TRUE;
                }

                break;
            default:
                break;
            } /* switch(line_number) */

        } /* for(;;) */

        fclose(setting);
    } /* if(setting != NULL) */

}
