#include "rufeol.h"

extern int integer_queue[INTEGER_QUEUE_SIZE];
extern int integer_queue_begin, integer_queue_end;
extern int integer_ring_count;

static int BELL = 0x07;
static int ERROR_CODE = 1;
static int LF_SIZE = 1;

BOOL buffer_begins_with_bell(char *p) {
    BOOL b = FALSE;
    char h;
    int item = integer_queue_begin;

    if(integer_queue[item] == BELL) {
        b = TRUE;
    }

    h = 0x00FF & integer_queue[item];
    *p = (char)h;
    return b;
}

BOOL setting(REMOVE_FROM_LINE *r, char *e, char *n, int b, int f, int u, BOOL g) {
    BOOL allocated = TRUE;
    int size;

    strcpy(r->file_name, e);
    strcpy(r->new_name, n);
    r->buffer_size = b;
    r->full_level = f;
    r->no_removing = g;
    r->unnecessary = u;

    r->whole = file_size(r->file_name);
    if(r->whole == FILE_ERROR) {
        r->buffer = NULL;
    } else {
        size = (int)r->whole;
        if(r->buffer_size > size) {
            r->buffer_size = size;
        }

        r->buffer = malloc(r->buffer_size);
        if(r->buffer == NULL) {
            fprintf(stderr, "malloc() error: %d bytes", r->buffer_size);
            allocated = FALSE;
        } else {
            r->combo = malloc(r->buffer_size);
            if(r->combo == NULL) {
                fprintf(stderr, "r->combo == NULL error: %d bytes", r->buffer_size);
                allocated = FALSE;
            }

        }

    }

    return allocated;
}

int insert_CRLF(REMOVE_FROM_LINE *r) {
    BOOL full = FALSE;
    char h;
    FILE *temporary_file = fopen(TEMPORARY_BINARY, "wb");
    int count = 0;
    int exit_code = 0;
    int k = 0;
    int c, m;
    unsigned char uc;
    FILE *binary_file = fopen(r->file_name, "rb");
    if(binary_file == NULL) {
        fprintf(stderr, "fopen() error-1: %s\n", r->file_name);
        return ERROR_CODE;
    }

/* ring buffer */
    integer_ring_count = r->unnecessary + LF_SIZE;
    prepare_ring_buffer(integer_ring_count, BELL);

    fread(r->buffer, 1, r->buffer_size, binary_file);
    for(m = 0; m < r->buffer_size; ++m) {
        uc = (unsigned char)r->buffer[m];
        c = (int)uc;
        if(c == 0x0A) {
            count += 1;
            if(count >= r->full_level) {
                full = TRUE;
            }

        }

        pop_frontal();
        push_back_integer(c);

        if(full) {
            r->combo[k++] = 0x0D;
            r->combo[k++] = 0x0A;
            prepare_ring_buffer(integer_ring_count, BELL);
            count = 0;
            full = FALSE;
        } else {
            if( buffer_begins_with_bell(&h) ) {
                continue; /* skipping 07h */
            } else {
                r->combo[k++] = h;
            }

        }

    }

    fwrite(r->combo, k, 1, temporary_file);
    fclose(temporary_file);
    fclose(binary_file);
    exit_code = rename_binary_file(r->new_name, r->no_removing);
    return exit_code;
}

int insert_LF(REMOVE_FROM_LINE *r) {
    BOOL full = FALSE;
    char h;
    FILE *temporary_file = fopen(TEMPORARY_BINARY, "wb");
    int count = 0;
    int exit_code = 0;
    int k = 0;
    int c, m;
    unsigned char uc;
    FILE *binary_file = fopen(r->file_name, "rb");
    if(binary_file == NULL) {
        fprintf(stderr, "fopen() error-2: %s\n", r->file_name);
        return ERROR_CODE;
    }

/* ring buffer */
    integer_ring_count = r->unnecessary + LF_SIZE;
    prepare_ring_buffer(integer_ring_count, BELL);

    fread(r->buffer, 1, r->buffer_size, binary_file);
    for(m = 0; m < r->buffer_size; ++m) {
        uc = (unsigned char)r->buffer[m];
        c = (int)uc;
        if(c == 0x0A) {
            count += 1;
            if(count >= r->full_level) {
                full = TRUE;
            }

        }

        pop_frontal();
        push_back_integer(c);

        if(full) {
            r->combo[k++] = 0x0A;
            prepare_ring_buffer(integer_ring_count, BELL);
            count = 0;
            full = FALSE;
        } else {
            if( buffer_begins_with_bell(&h) ) {
                continue; /* skipping 07h */
            } else {
                r->combo[k++] = h;
            }

        }

    }

    fwrite(r->combo, k, 1, temporary_file);
    fclose(temporary_file);
    fclose(binary_file);
    exit_code = rename_binary_file(r->new_name, r->no_removing);
    return exit_code;
}

int rename_binary_file(const char *name, BOOL no_removing) {
    char c[BLOCK_SIZE];
    int exit_code = 0;
    int size;
    FILE *temporary = fopen(TEMPORARY_BINARY, "rb");
    FILE *recent = fopen(name, "wb");
    if(recent == NULL) {
        recent = fopen(RUFEOL_TEXT, "wb");
    }

    if(temporary == NULL) {
        exit_code = FILE_ERROR_CODE;
    } else {
        do {
            size = fread(c, 1, BLOCK_SIZE, temporary);
            fwrite(c, size, 1, recent);
        } while(size == BLOCK_SIZE);

        fclose(recent);
        fclose(temporary);

        if(! no_removing) {
            remove(TEMPORARY_BINARY);
        }

    }

    return exit_code;
}

long file_size(char *file_name) {
    long size = FILE_ERROR;
    FILE *p = fopen(file_name, "rb");
    if(p == NULL) {
        error_handler("fopen() error-3:", file_name);
        return size;
    }

    fseek(p, 0, END_OF_FILE);
    size = ftell(p);
    if( size < 0L ) {
        error_handler("ftell() error:", file_name);
    }

    fclose(p);
    return size;
}

void error_handler(const char *p, char *q) {
    FILE *file = fopen(ERROR_TEXT, "wt");
    fprintf(file, "%s %s", p, q);;
    fclose(file);
}

void release(REMOVE_FROM_LINE *r) {
    free(r->buffer);
    free(r->combo);
}
