#include "sutai.h"

DOUBLE_UNION ZEROED;
static int ERROR_CODE = 1;

BOOL producing(char *new_name, char *records_list, int end, int line_count, int record_size, BOOL carriage_return) {
    BOOL b = TRUE;
    char *p;
    char *q;
    char new_line[3] = {0x0A, 0, 0};
    int i;
    int offset = 0;
    FILE *file = fopen(new_name, "wb");
    if(file == NULL) {
        b = FALSE;
        error_handler("fopen() error-1:", new_name);
        file = fopen(SUTAI_TEXT, "wb");
    }

    if(carriage_return) {
        new_line[0] = 0x0D;
        new_line[1] = 0x0A;
    }

    for(i = 0; i < line_count; i++) {
        p = records_list + offset + end;
        q = strchr(p, '\n');
        if(q != NULL) {
            *q = '\0';
        }

        fputs(p, file);
        fputs(new_line, file);

        offset += record_size;
    }

    fclose(file);
    return b;
}

int arranging_to_produce_text(char *file_name, char *new_name, int line_count, int manager, int record_size, BOOL add_ID, BOOL carriage_return, BOOL case_sensitive, BOOL descending) {
    BOOL result = TRUE;
    char *p;
    char *records_list;
    int end_of_summary = END;
    int exit_code = 0;
    int file_name_size, i;
    int list_size = record_size * line_count;
    int offset = 0;
    FILE *binary_file = fopen(file_name, "rb");
    if(binary_file == NULL) {
        fprintf(stderr, "fopen() error-2: %s\n", file_name);
        return ERROR_CODE;
    }

    records_list = malloc(list_size);
    if(records_list == NULL) {
        fprintf(stderr, "malloc() error-1: %d bytes\n", list_size);
        return ERROR_CODE;
    }

    for(i = 0; i < line_count; i++) {
        p = records_list + offset;
        fread(p, record_size, 1, binary_file);

        offset += record_size;
    }

    fclose(binary_file);

    if(descending) {
        switch(manager) {
        case PHRASES:
            if(case_sensitive) {
                qsort(records_list, line_count, record_size, compare_phrases_descending);
            } else {
                qsort(records_list, line_count, record_size, case_insensitive_comparison_descending);
            }

            break;
        case INTEGERS:
            qsort(records_list, line_count, record_size, compare_integers_descending);
            break;
        case FLOATING_POINT_NUMBERS:
            qsort(records_list, line_count, record_size, compare_doubles_descending);
            break;
        default:
            qsort(records_list, line_count, record_size, compare_phrases_descending);
            break;
        }

    } else {
        switch(manager) {
        case PHRASES:
            if(case_sensitive) {
                qsort(records_list, line_count, record_size, compare_phrases);
            } else {
                qsort(records_list, line_count, record_size, case_insensitive_comparison);
            }

            break;
        case INTEGERS:
            qsort(records_list, line_count, record_size, compare_integers);
            break;
        case FLOATING_POINT_NUMBERS:
            qsort(records_list, line_count, record_size, compare_doubles);
            break;
        default:
            qsort(records_list, line_count, record_size, compare_phrases);
            break;
        }

    }

    if(add_ID) {
        end_of_summary = END_ID;
    }

    file_name_size = strlen(new_name);
    if(file_name_size == 0) {
        display_records(records_list, end_of_summary, line_count, record_size);
    } else {
        result = producing(new_name, records_list, end_of_summary, line_count, record_size, carriage_return);
        exit_code = result ? 0 : ERROR_CODE;
    }

    free(records_list);
    return exit_code;
}

int arranging_unique_lines_to_produce_text(char *file_name, char *new_name, int line_count, int manager, int record_size, BOOL add_ID, BOOL carriage_return, BOOL case_sensitive, BOOL descending) {
    BOOL result = TRUE;
    char *buffer;
    char *p;
    char *q;
    char *records_list;
    char combo[MAXIMUM_SIZE_OF_ENCODED_PATTERN];
    char previous[MAXIMUM_SIZE_OF_ENCODED_PATTERN];
    DOUBLE_UNION u = ZEROED;
    DOUBLE_UNION *n = &u;
    char *n_c = & n->c[0];
    int combo_size = sizeof(combo);
    int end_of_summary = END;
    int exit_code = 0;
    int file_name_size, i;
    int offset = 0;
    int list_size = record_size * line_count;
    int unique_lines = 0;
    FILE *binary_file = fopen(file_name, "rb");
    if(binary_file == NULL) {
        fprintf(stderr, "fopen() error-3: %s\n", file_name);
        return ERROR_CODE;
    }

    records_list = malloc(list_size);
    if(records_list == NULL) {
        fprintf(stderr, "malloc() error-2: %d bytes\n", list_size);
        return ERROR_CODE;
    }

    buffer = malloc(list_size);
    if(buffer == NULL) {
        fprintf(stderr, "malloc() error-3: %d bytes\n", list_size);
        return ERROR_CODE;
    }

    for(i = 0; i < line_count; ++i) {
        p = records_list + offset;
        fread(p, record_size, 1, binary_file);

        offset += record_size;
    }

    fclose(binary_file);

    if(descending) {
        switch(manager) {
        case PHRASES:
            if(case_sensitive) {
                qsort(records_list, line_count, record_size, compare_phrases_descending);
            } else {
                qsort(records_list, line_count, record_size, case_insensitive_comparison_descending);
            }

            break;
        case INTEGERS:
            qsort(records_list, line_count, record_size, compare_integers_descending);
            break;
        case FLOATING_POINT_NUMBERS:
            qsort(records_list, line_count, record_size, compare_doubles_descending);
            break;
        default:
            qsort(records_list, line_count, record_size, compare_phrases_descending);
            break;
        }

    } else {
        switch(manager) {
        case PHRASES:
            if(case_sensitive) {
                qsort(records_list, line_count, record_size, compare_phrases);
            } else {
                qsort(records_list, line_count, record_size, case_insensitive_comparison);
            }

            break;
        case INTEGERS:
            qsort(records_list, line_count, record_size, compare_integers);
            break;
        case FLOATING_POINT_NUMBERS:
            qsort(records_list, line_count, record_size, compare_doubles);
            break;
        default:
            qsort(records_list, line_count, record_size, compare_phrases);
            break;
        }

    }

    memset(combo, 0, combo_size);
    strcpy(previous, "\x80"); // previous differs from combo.
    memcpy(buffer, records_list, list_size);
    p = records_list;
    q = buffer;
    for(i = 0; i < line_count; ++i) {
        u = ZEROED;
        memcpy(n_c, q, END_ID);
        run_length_encoding(n, combo, combo_size);

        if( strcmp(previous, combo) ) { // previous differs from combo.
            memcpy(p, q, record_size);
            p += record_size;
            unique_lines += 1;
            strcpy(previous, combo);
        }

        q += record_size;
    }

    if(add_ID) {
        end_of_summary = END_ID;
    }

    file_name_size = strlen(new_name);
    if(file_name_size == 0) {
        display_records(records_list, end_of_summary, unique_lines, record_size);
    } else {
        result = producing(new_name, records_list, end_of_summary, unique_lines, record_size, carriage_return);
        exit_code = result ? 0 : ERROR_CODE;
    }

    free(buffer);
    free(records_list);
    return exit_code;
}

void display_records(char *records_list, int end, int line_count, int record_size) {
    char *p;
    int offset = 0;
    int i;

    for(i = 0; i < line_count; i++) {
        p = records_list + offset + end;
        fputs(p, stdout);

        offset += record_size;
    }

}

void error_handler(const char *p, char *q) {
    FILE *file = fopen(ERROR_TEXT, "wt");
    fprintf(file, "%s %s", p, q);;
    fclose(file);
}

