#include "line.h"

// maximum_LF_to_LF_span() counts lines of the file to produce line_count.

BOOL reading_fields(char *file_name, int buffer_size, int line_count, int point, int separator, BOOL measure) {
    BOOL b = FALSE;
    char *data;
    char *p;
    int field_count, field_length;
    int n = 0;
    int size;
    FILE *sjis_file = fopen(file_name, "rt");
    char *buffer = malloc(buffer_size);
    if(buffer == NULL) {
        fprintf(stderr, "malloc() error-6: %d bytes\n", buffer_size);
        return FALSE;
    }

    for(;;) {
        p = fgets(buffer, buffer_size, sjis_file); // buffer includes 0Ah.
        if(p == NULL) {
            break;
        }

        n += 1;
        if(n == point) {
            b = TRUE;
            break;
        }

    }

    fclose(sjis_file);
    size = strlen(buffer);
    data = malloc( size + 1 );
    strcpy(data, buffer);

    if(measure) {
        fprintf(stdout, "%s: %d ", file_name, line_count);
        if(line_count == 1) {
            fprintf(stdout, "line\n");
        } else {
            fprintf(stdout, "lines\n");
        }

        field_length = maximum_separator_to_separator_span(&field_count, buffer, size, separator);
        field_length -= SEPARATOR_SIZE;
        fprintf(stdout, "maximum separator-to-separator span of %d bytes (line-%d)\n", field_length, point);
    }

    fprintf(stdout, "%s", data);
    free(buffer);
    free(data);
    return b;
}

BOOL reading_lines(char *file_name, int buffer_size, int line_count, int line_number, int maximum_line_length, int point, BOOL measure) {
    BOOL b = FALSE;
    char *p;
    int n = 0;
    FILE *sjis_file = fopen(file_name, "rt");
    char *buffer = malloc(buffer_size);
    if(buffer == NULL) {
        fprintf(stderr, "malloc() error-7: %d bytes\n", buffer_size);
        return FALSE;
    }

    for(;;) {
        p = fgets(buffer, buffer_size, sjis_file); // buffer includes 0Ah.
        if(p == NULL) {
            break;
        }

        n += 1;
        if(n == point) {
            b = TRUE;
            break;
        }

    }

    fclose(sjis_file);

    if(measure) {
        fprintf(stdout, "%s: %d ", file_name, line_count);
        if(line_count == 1) {
            fprintf(stdout, "line\n");
        } else {
            fprintf(stdout, "lines\n");
        }

        fprintf(stdout, "maximum LF-to-LF span of %d bytes (line-%d)\n", maximum_line_length, line_number);
    }

    fprintf(stdout, "%s", buffer);
    free(buffer);
    return b;
}

int maximum_LF_to_LF_span(int *line_count, int *line_number, BOOL *carriage_return, char *buffer, int buffer_size) { // maximum_LF_to_LF_span() counts lines of the file to produce line_count.
    int c, m;
    int count = 0;
    int k;
    int max = 0;
    int new_line_length = 1;
    int trace = NOT_FOUND;
    unsigned char uc;

    *carriage_return = FALSE;
    for(k = 0; k < buffer_size && trace == NOT_FOUND; ++k) {
        uc = (unsigned char)buffer[k];
        c = (int)uc;

        switch(c) {
        case 0x0A:
            count += 1;

            if(carriage_return) {
                max = k - 1 + new_line_length;
            } else {
                max = k + new_line_length;
            }

            *line_number = count;
            trace = k; // trace != NOT_FOUND
            break;
        case 0x0D:
            *carriage_return = TRUE;
            new_line_length = 2;
            break;
        default:
            break;
        }

    }

    for( ; k <= buffer_size; ++k) {
        uc = (unsigned char)buffer[k];
        c = (int)uc;

        switch(c) {
        case 0x0A:
            count += 1;
            m = k - trace;
            if(m > max) {
                max = m;
                *line_number = count;
            }

            trace = k;
        default:
            break;
        }

    }

    *line_count = count;
    return max;
}

int maximum_separator_to_separator_span(int *p, char *buffer, int buffer_size, int separator) {
    int c, m;
    int count = 0;
    int k;
    int max = 0;
    int trace = NOT_FOUND;
    unsigned char uc;

    for(k = 0; k < buffer_size; ++k) {
        uc = (unsigned char)buffer[k];
        c = (int)uc;

        if(c == separator) {
            count += 1;
            max = k + SEPARATOR_SIZE;
            trace = k;
            break;
        }

    }

    for( ; k < buffer_size; ++k) {
        uc = (unsigned char)buffer[k];
        c = (int)uc;

        if(c == separator) {
            count += 1;
            m = k - trace;
            if(m > max) {
                max = m;
            }

            trace = k;
        }

    }

    *p = count;
    return max;
}

long file_size(char *file_name) {
    long size = FILE_ERROR;
    FILE *p = fopen(file_name, "rb");
    if(p == NULL) {
        fprintf(stderr, "fopen() error-5: %s\n", file_name);
        return size;
    }

    fseek(p, 0, END_OF_FILE);
    size = ftell(p);
    if(size < 0L) {
        fprintf(stderr, "ftell() error: %ld\n", size);
        return size;
    }

    fclose(p);
    return size;
}

